adtestsim=function (x, R = 1000, locscatt = "standard") 
{
    if (R < 1) 
        stop("choose a higher value for R")
    if (R < 50) 
        warnings("maybe, the estimation of the p-value(s) is not accurate; choose a higher value for R")
    cv <- function(x, type) {
        classical <- function(x) {
            if ((length(dim(x)) < 1) | is.vector(x)) {
                list(mean = mean(x, na.rm = TRUE), varmat = var(x, 
                  na.rm = TRUE))
            }
            else {
                list(mean = colMeans(x, na.rm = TRUE), varmat = cov(x))
            }
        }
        robust <- function(x) {
            if ((length(dim(x)) < 1) | is.vector(x)) {
                list(mean = median(x), varmat = mad(x)^2)
            }
            else {
                v <- covMcd(x)
                list(mean = v$center, varmat = v$cov)
            }
        }
        switch(type, standard = classical(x), robust = robust(x))
    }
    centre <- function(x, type) {
        switch(type, mean = mean(x), median = median(x), trimmed = mean(x, 
            trim = 0.1))
    }
    if (locscatt == "standard") 
        location <- "mean"
    else location <- "median"
    if ((length(dim(x)) < 1) | is.vector(x)) {
        x <- as.vector(x)
        n <- length(x)
        if (n < 8) 
            stop("sample size must be greater than 7")
        stat = function(x, N = n, location = "mean") {
            x <- sort(x[complete.cases(x)])
            estCv <- cv(x, locscatt)
            p <- pnorm((x - estCv$mean)/sqrt(estCv$varmat))
            h <- (2 * seq(1:N) - 1) * (log(p) + log(1 - rev(p)))
            A <- (25/N^2 - 4/N - 1) * (centre(h, location) + 
                N)
        }
        A = stat(x, location = location)
        estCv <- cv(x, locscatt)
        mv <- estCv$mean
        varmat <- estCv$varmat
        n = length(c(x))
        p <- sapply(X = 1:R, FUN = function(X, ...) {
            stat(rnorm(n, mv, sqrt(varmat)), location = location)
        })
        pvalue = mean(p > 0.656)
        RVAL <- list(statistic = c(A = A), method = "A-D univariate normality test", 
            p.value = pvalue)
    }
    class(RVAL) <- "htest"
    return(RVAL)
}

